@@ -110,6 +110,12 @@ static inline void freecode(ident &id)
     }
 }
 
+struct nullval : tagval
+{
+    nullval() { setnull(); }
+} nullret;
+tagval *commandret = &nullret;
+
 void clear_command()
 {
     enumerate(idents, ident, i,
@@ -134,9 +140,9 @@ void clearoverride(ident &i)
                 if(!i.val.s[0]) break;
                 delete[] i.val.s;
             }
+            freecode(i);
             i.valtype = VAL_STR;
             i.val.s = newstring("");
-            i.code = NULL;
             break;
         case ID_VAR:
             *i.storage.i = i.overrideval.i;
@@ -221,8 +227,12 @@ static struct identlink
 
 VAR(dbgalias, 0, 4, 1000);
 
+static int nodebug = 0;
+
 static void debugcode(const char *fmt, ...)
 {
+    if(nodebug) return;
+
     va_list args;
     va_start(args, fmt);
     conoutfv(CON_ERROR, fmt, args);
@@ -240,6 +250,8 @@ static void debugcode(const char *fmt, ...)
     }
 }
 
+ICOMMAND(nodebug, "e", (uint *body), { nodebug++; executeret(body, *commandret); nodebug--; });
+
 void addident(ident *id)
 {
     addident(*id);
@@ -272,15 +284,25 @@ ICOMMAND(push, "rte", (ident *id, tagval *v, uint *code),
     pusharg(*id, *v, stack);
     v->type = VAL_NULL;
     id->flags &= ~IDF_UNKNOWN;
-    execute(code);
+    executeret(code, *commandret);
     poparg(*id);
 });
 
+static inline bool isinteger(const char *c)
+{
+    return isdigit(c[0]) || ((c[0]=='+' || c[0]=='-' || c[0]=='.') && isdigit(c[1]));
+}
+
 ident *newident(const char *name, int flags)
 {
     ident *id = idents.access(name);
     if(!id)
     {
+        if(isinteger(name))
+        {
+            debugcode("integer %s is not a valid identifier name", name);
+            return newident("//dummy", IDF_UNKNOWN);
+        }
         ident init(ID_ALIAS, newstring(name), flags);
         id = addident(init);
     }
@@ -317,6 +339,11 @@ static void setalias(const char *name, tagval &v)
             freearg(v);
         }
     }
+    else if(isinteger(name))
+    {
+        debugcode("cannot alias integer %s", name);
+        freearg(v);
+    }
     else
     {
         ident def(ID_ALIAS, newstring(name), v, identflags);
@@ -424,6 +451,16 @@ int getvarmax(const char *name)
     GETVAR(id, name, 0);
     return id->maxval;
 }
+float getfvarmin(const char *name)
+{
+    _GETVAR(id, ID_FVAR, name, 0);
+    return id->minvalf;
+}
+float getfvarmax(const char *name)
+{
+    _GETVAR(id, ID_FVAR, name, 0);
+    return id->maxvalf;
+}
 bool identexists(const char *name) { return idents.access(name)!=NULL; }
 ident *getident(const char *name) { return idents.access(name); }
 
@@ -622,11 +659,6 @@ char *conc(tagval *v, int n, bool space, const char *prefix = NULL)
     return buf;
 }
 
-static inline bool isinteger(char *c)
-{
-    return isdigit(c[0]) || ((c[0]=='+' || c[0]=='-' || c[0]=='.') && isdigit(c[1]));
-}
-
 static inline void skipcomments(const char *&p)
 {
     for(;;)
@@ -1155,6 +1187,7 @@ static void compilestatements(vector<uint> &code, const char *&p, int rettype, i
             delete[] idname;
         }
     endstatement:
+        if(more) while(compilearg(code, p, VAL_ANY)) code.add(CODE_POP);
         p += strcspn(p, ")];/\n\0");
         int c = *p++;
         switch(c)
@@ -1226,12 +1259,6 @@ typedef void (__cdecl *comfun7)(void *, void *, void *, void *, void *, void *,
 typedef void (__cdecl *comfun8)(void *, void *, void *, void *, void *, void *, void *, void *);
 typedef void (__cdecl *comfunv)(tagval *, int);
 
-struct nullval : tagval
-{
-    nullval() { setnull(); }
-} nullret;
-tagval *commandret = &nullret;
-
 struct argidentsvector : vector<ident *>
 {
     argidentsvector()
@@ -1248,7 +1275,7 @@ static const uint *runcode(const uint *code, tagval &result)
 {
     ident *id = NULL;
     int numargs = 0;
-    tagval args[MAXARGS], *prevret = commandret;
+    tagval args[MAXARGS+1], *prevret = commandret;
     result.setnull();
     commandret = &result;
     for(;;)
@@ -1257,6 +1284,9 @@ static const uint *runcode(const uint *code, tagval &result)
         switch(op&0xFF)
         {
             case CODE_NOP: continue;
+            case CODE_POP:
+                freearg(args[--numargs]);
+                continue;
             case CODE_ENTER:
                 code = runcode(code, args[numargs++]);
                 continue;
@@ -1638,6 +1668,7 @@ char *executeret(const char *p)
 {
     tagval result;
     executeret(p, result);
+    if(result.type == VAL_NULL) return NULL;
     forcestr(result);
     return result.s;
 }
@@ -1700,9 +1731,9 @@ bool execfile(const char *cfgfile, bool msg)
 }
 
 #ifndef STANDALONE
-int sortidents(ident **x, ident **y)
+bool sortidents(ident *x, ident *y)
 {
-    return strcmp((*x)->name, (*y)->name);
+    return strcmp(x->name, y->name) < 0;
 }
 
 void writeescapedstring(stream *f, const char *s)
@@ -1719,6 +1750,23 @@ void writeescapedstring(stream *f, const char *s)
     f->putchar('"');
 }
 
+static bool validatealias(const char *s)
+{
+    int parens = 0, braks = 0;
+    for(; *s; s++) switch(*s)
+    {
+        case '(': parens++; break;
+        case ')': if(!parens) return false; parens--; break;
+        case '[': braks++; break;
+        case ']': if(!braks) return false; braks--; break;
+        case '"': s = parsestring(s + 1); if(*s != '"') return false; break;
+        case '/': if(s[1] == '/') return false; break;
+        case '\f': return false;
+    }
+    if(braks || parens) return false;
+    return true;
+}
+
 void writecfg(const char *name)
 {
     defformatstring(confname)("config_%s.cfg", game::gameident());
@@ -1726,8 +1774,8 @@ void writecfg(const char *name)
 
     if(!f) return;
     f->printf("// automatically written on exit, DO NOT MODIFY\n// delete this file to have data/defaults.cfg and data/%s/defaults.cfg overwrite these settings\n// modify settings in game, or put settings in %s to override anything\n\n", game::gameident(), game::autoexec());
-    f->printf("if (|| (= $ignoreconfver 1) (= $confver %i)) [\n", confver);
-    f->printf("\n");
+    f->printf("if (&& (! $ignoreconfver) (!= $confver %i)) [\n\tsleep 500 [showgui keepconf]; setdefaults\n] [\n\n", confver);
+
     writecrosshairs(f);
     vector<ident *> ids;
     enumerate(idents, ident, id, ids.add(&id));
@@ -1752,6 +1800,7 @@ void writecfg(const char *name)
         {
         case VAL_STR:
             if(!id.val.s[0]) break;
+            if(!validatealias(id.val.s)) { f->printf("\"%s\" = ", id.name); writeescapedstring(f, id.val.s); f->putchar('\n'); break; }
         case VAL_FLOAT:
         case VAL_INT:
             f->printf("\"%s\" = [%s]\n", id.name, id.getstr()); break;
@@ -1759,7 +1808,7 @@ void writecfg(const char *name)
     }
     f->printf("\n");
     writecompletions(f);
-    f->printf("] [sleep 500 [showgui keepconf]; setdefaults]\n\n");
+    f->printf("\n]\n\n");
     game::writeclientinfo(f);
     delete f;
 }
@@ -1807,7 +1856,7 @@ static inline bool getbool(const tagval &v)
         case VAL_NULL: return false;
         case VAL_FLOAT: return v.f!=0;
         case VAL_INT: return v.i!=0;
-        case VAL_STR: return !isinteger(v.s) || parseint(v.s);
+        case VAL_STR: case VAL_MACRO: return v.s[0] && (!isinteger(v.s) || parseint(v.s));
         default: return false;
     }
 }
@@ -1821,7 +1870,11 @@ ICOMMAND(loop, "rie", (ident *id, int *n, uint *body),
     identstack stack;
     loopi(*n)
     {
-        if(i) id->val.i = i;
+        if(i)
+        {
+            if(id->valtype != VAL_INT) { if(id->valtype == VAL_STR) delete[] id->val.s; freecode(*id); id->valtype = VAL_INT; }
+            id->val.i = i;
+        }
         else
         {
             tagval zero;
@@ -1839,7 +1892,11 @@ ICOMMAND(loopwhile, "riee", (ident *id, int *n, uint *cond, uint *body),
     identstack stack;
     loopi(*n)
     {
-        if(i) id->val.i = i;
+        if(i)
+        {
+            if(id->valtype != VAL_INT) { if(id->valtype == VAL_STR) delete[] id->val.s; freecode(*id); id->valtype = VAL_INT; }
+            id->val.i = i;
+        }
         else
         {
             tagval zero;
@@ -1952,6 +2009,26 @@ void substr(char *s, int *start, char *count)
     commandret->setstr(newstring(&s[offset], count[0] ? clamp(parseint(count), 0, len - offset) : len - offset));
 }
 
+void sublist(char *s, int *start, char *count)
+{
+    int offset = max(*start, 0), len = count[0] ? max(parseint(count), 0) : -1;
+    loopi(offset)
+    {
+        whitespaceskip;
+        elementskip;
+        if(!*s) break;
+    }
+    if(len < 0) { commandret->setstr(newstring(s)); return; }
+    const char *e = s;
+    loopi(len)
+    {
+        whitespaceskip;
+        elementskip;
+        if(!*s) break;
+    }
+    commandret->setstr(newstring(e, s - e));
+}
+
 void getalias_(char *s)
 {
     result(getalias(s));
@@ -1969,18 +2046,14 @@ COMMAND(concatword, "V");
 COMMAND(format, "V");
 COMMAND(at, "si");
 COMMAND(substr, "sis");
+COMMAND(sublist, "sis");
 ICOMMAND(listlen, "s", (char *s), intret(listlen(s)));
 COMMANDN(getalias, getalias_, "s");
+ICOMMAND(getvarmin, "s", (char *s), intret(getvarmin(s)));
+ICOMMAND(getvarmax, "s", (char *s), intret(getvarmax(s)));
+ICOMMAND(getfvarmin, "s", (char *s), floatret(getfvarmin(s)));
+ICOMMAND(getfvarmax, "s", (char *s), floatret(getfvarmax(s)));
 ICOMMAND(isdir, "s", (char *dir), intret(isdir(dir)));
-ICOMMAND(getvar, "r", (ident *id),
-    switch(id->type)
-    {
-        case ID_VAR: intret(*id->storage.i); break;
-        case ID_FVAR: floatret(*id->storage.f); break;
-        case ID_SVAR: result(*id->storage.s); break;
-        case ID_ALIAS: result(id->val.s); break;
-    }
-)
 
 void looplist(ident *id, const char *list, const uint *body, bool search)
 {
@@ -1998,7 +2071,9 @@ void looplist(ident *id, const char *list, const uint *body, bool search)
         char *val = newstring(start, end-start);
         if(n++)
         {
-            delete[] id->val.s;
+            if(id->type == VAL_STR) delete[] id->val.s;
+            else id->valtype = VAL_STR;
+            freecode(*id);
             id->val.s = val;
         }
         else
@@ -2104,7 +2179,8 @@ ICOMMAND(loopfiles, "rsse", (ident *id, char *dir, char *ext, uint *body),
         if(redundant) { delete[] file; continue; }
         if(i)
         {
-            if(id->valtype == VAL_STR) delete[] id->val.s;
+            if(id->type == VAL_STR) delete[] id->val.s;
+            else id->valtype = VAL_STR;
             id->val.s = file;
         }
         else
@@ -2316,8 +2392,6 @@ char *strreplace(const char *s, const char *oldval, const char *newval)
 ICOMMAND(strreplace, "sss", (char *s, char *o, char *n), commandret->setstr(strreplace(s, o, n)));
 
 #ifndef STANDALONE
-ICOMMAND(getmillis, "i", (int *total), intret(*total ? totalmillis : lastmillis));
-
 struct sleepcmd
 {
     int delay, millis, flags;
