@@ -85,7 +85,7 @@ namespace mpr
         quat orient;
         float zmargin;
 
-        EntOBB(physent *ent, float zmargin = 0) : ent(ent), orient(vec(0, 0, 1), (ent->yaw + 180)*RAD), zmargin(zmargin) {}
+        EntOBB(physent *ent, float zmargin = 0) : ent(ent), orient(vec(0, 0, 1), ent->yaw*RAD), zmargin(zmargin) {}
 
         vec center() const { vec o(ent->o); o.z += (ent->aboveeye - ent->eyeheight - zmargin)/2; return o; }
 
@@ -197,7 +197,10 @@ namespace mpr
         vec o, radius;
         quat orient;
 
-        ModelOBB(const vec &o, const vec &radius, float yaw) : o(o), radius(radius), orient(vec(0, 0, 1), yaw*RAD) {}
+        ModelOBB(const vec &ent, const vec &center, const vec &radius, float yaw) : o(ent), radius(radius), orient(vec(0, 0, 1), yaw*RAD) 
+        {
+            o.add(orient.rotate(center));
+        }
 
         vec center() const { return o; }
 
@@ -234,7 +237,10 @@ namespace mpr
         vec o, radius;
         quat orient;
 
-        ModelEllipse(const vec &o, const vec &radius, float yaw) : o(o), radius(radius), orient(vec(0, 0, 1), yaw*RAD) {}
+        ModelEllipse(const vec &ent, const vec &center, const vec &radius, float yaw) : o(ent), radius(radius), orient(vec(0, 0, 1), yaw*RAD) 
+        {
+            o.add(orient.rotate(center));
+        }
 
         vec center() const { return o; }
 
