@@ -9,12 +9,7 @@ cubeext *newcubeext(cube &c)
 {
     if(c.ext) return c.ext;
     c.ext = new cubeext;
-    c.ext->material = MAT_AIR;
-    c.ext->visible = 0;
-    c.ext->merged = 0;
-    c.ext->mergeorigin = 0;
     c.ext->va = NULL;
-    c.ext->clip = NULL;
     c.ext->surfaces = NULL;
     c.ext->normals = NULL;
     c.ext->ents = NULL;
@@ -23,18 +18,18 @@ cubeext *newcubeext(cube &c)
     return c.ext;
 }
 
-cube *newcubes(uint face)
+cube *newcubes(uint face, int mat)
 {
     cube *c = new cube[8];
     loopi(8)
     {
         c->children = NULL;
         c->ext = NULL;
+        c->visible = 0;
+        c->merged = 0;
         setfaces(*c, face);
-        loopl(6)
-        {
-            c->texture[l] = DEFAULT_GEOM;
-        }
+        loopl(6) c->texture[l] = DEFAULT_GEOM;
+        c->material = mat;
         c++;
     }
     allocnodes++;
@@ -63,15 +58,15 @@ void freecubeext(cube &c)
 
 void discardchildren(cube &c, bool fixtex, int depth)
 {
+    c.visible = 0;
+    c.merged = 0;
     if(c.ext)
     {
         if(c.ext->va) destroyva(c.ext->va);
         c.ext->va = NULL;
-        c.ext->merged = 0;
         c.ext->tjoints = -1;
         freesurfaces(c);
         freenormals(c);
-        freeclipplanes(c);
         freeoctaentities(c);
         freemergeinfo(c);
         freecubeext(c);
@@ -126,8 +121,8 @@ static inline void setcubevector(cube &c, int i, const ivec &p)
 
 void optiface(uchar *p, cube &c)
 {
-    loopi(4) if(edgeget(p[i], 0)!=edgeget(p[i], 1)) return;
-    emptyfaces(c);
+    uint f = *(uint *)p;
+    if(((f>>4)&0x0F0F0F0FU) == (f&0x0F0F0F0FU)) emptyfaces(c);
 }
 
 void printcube()
@@ -170,11 +165,15 @@ void validatec(cube *c, int size)
         }
         else
         {
-            loopj(3) optiface((uchar *)&(c[i].faces[j]), c[i]);
-            loopj(12)
-                if(edgeget(c[i].edges[j], 1)>8 ||
-                   edgeget(c[i].edges[j], 1)<edgeget(c[i].edges[j], 0))
+            loopj(3)
+            {
+                uint f = c[i].faces[j], e0 = f&0x0F0F0F0FU, e1 = (f>>4)&0x0F0F0F0FU;
+                if(e0 == e1 || ((e1+0x07070707U)|(e1-e0))&0xF0F0F0F0U)
+                {
                     emptyfaces(c[i]);
+                    break;
+                }
+            }
         }
     }
 }
@@ -219,7 +218,7 @@ int lookupmaterial(const vec &v)
         scale--;
         c = &c->children[octastep(o.x, o.y, o.z, scale)];
     }
-    return c->ext ? c->ext->material : MAT_AIR;
+    return c->material;
 }
 
 cube *neighbourstack[32];
@@ -326,67 +325,68 @@ bool subdividecube(cube &c, bool fullcheck, bool brighten)
     if(c.ext && c.ext->surfaces) freesurfaces(c);
 	if(isempty(c) || isentirelysolid(c))
     {
-        int mat = c.ext ? c.ext->material : MAT_AIR;
-		c.children = newcubes(isempty(c) ? F_EMPTY : F_SOLID);
+		c.children = newcubes(isempty(c) ? F_EMPTY : F_SOLID, c.material);
         loopi(8)
         {
             loopl(6) c.children[i].texture[l] = c.texture[l];
-            if(mat!=MAT_AIR) ext(c.children[i]).material = mat;
             if(brighten && !isempty(c)) brightencube(c.children[i]);
         }
         return true;
     }
-    cube *ch = c.children = newcubes(F_SOLID);
-    bool perfect = true, p1, p2;
-    int mat = c.ext ? c.ext->material : MAT_AIR;
+    cube *ch = c.children = newcubes(F_SOLID, c.material);
+    bool perfect = true;
     ivec v[8];
     loopi(8)
     {
         getcubevector(c, i, v[i]);
         v[i].mul(2);
-        if(mat!=MAT_AIR) ext(ch[i]).material = mat;
     }
 
     loopj(6)
     {
-        int d = dimension(j);
-        int z = dimcoord(j);
+        int d = dimension(j), z = dimcoord(j);
+        const ivec &v00 = v[octaindex(d, 0, 0, z)],
+                   &v10 = v[octaindex(d, 1, 0, z)],
+                   &v01 = v[octaindex(d, 0, 1, z)],
+                   &v11 = v[octaindex(d, 1, 1, z)];
         int e[3][3];
-        ivec *v1[2][2];
-        loop(y, 2) loop(x, 2)
-            v1[x][y] = v+octaindex(d, x, y, z);
-
-        loop(y, 3) loop(x, 3)       // gen edges
+        // corners
+        e[0][0] = v00[d];
+        e[0][2] = v01[d];
+        e[2][0] = v10[d];
+        e[2][2] = v11[d];
+        // edges
+        e[0][1] = midedge(v00, v01, C[d], d, perfect);
+        e[1][0] = midedge(v00, v10, R[d], d, perfect);
+        e[1][2] = midedge(v11, v01, R[d], d, perfect);
+        e[2][1] = midedge(v11, v10, C[d], d, perfect);
+        // center
+        bool p1 = perfect, p2 = perfect;
+        int c1 = midedge(v00, v11, R[d], d, p1);
+        int c2 = midedge(v01, v10, R[d], d, p2);
+        if(z ? c1 > c2 : c1 < c2)
         {
-            if(x==1 && y==1)        // center
-            {
-                int c1 = midedge(*v1[0][0], *v1[1][1], R[d], d, p1 = perfect);
-                int c2 = midedge(*v1[0][1], *v1[1][0], R[d], d, p2 = perfect);
-                e[x][y] = z ? max(c1,c2) : min(c1,c2);
-                perfect = e[x][y]==c1 ? p1 : p2;
-            }
-            else if(((x+y)&1)==0)   // corner
-                e[x][y] = (*v1[x>>1][y>>1])[d];
-            else                    // edge
-            {
-                int a = min(x, y), b = x&1;
-                e[x][y] = midedge(*v1[a][a], *v1[a^b][a^(1-b)], x==1?R[d]:C[d], d, perfect);
-            }
+            e[1][1] = c1;
+            perfect = p1 && (c1 == c2 || v00[C[d]] + v11[C[d]] == 16);
+        }
+        else
+        {
+            e[1][1] = c2;
+            perfect = p2 && (c1 == c2 || v01[C[d]] + v10[C[d]] == 16);
         }
 
         loopi(8)
         {
-            ivec o(i);
             ch[i].texture[j] = c.texture[j];
-            loop(y, 2) loop(x, 2) // assign child edges
-            {
-                int ce = clamp(e[x+((i>>R[d])&1)][y+((i>>C[d])&1)] - ((i>>D[d])&1)*8, 0, 8);
-                edgeset(cubeedge(ch[i], d, x, y), z, ce);
-            }
+            int rd = (i>>R[d])&1, cd = (i>>C[d])&1, dd = (i>>D[d])&1;
+            edgeset(cubeedge(ch[i], d, 0, 0), z, clamp(e[rd][cd] - dd*8, 0, 8));
+            edgeset(cubeedge(ch[i], d, 1, 0), z, clamp(e[1+rd][cd] - dd*8, 0, 8));
+            edgeset(cubeedge(ch[i], d, 0, 1), z, clamp(e[rd][1+cd] - dd*8, 0, 8));
+            edgeset(cubeedge(ch[i], d, 1, 1), z, clamp(e[1+rd][1+cd] - dd*8, 0, 8));
         }
     }
 
-    validatec(ch, worldsize);
+    validatec(ch);
     if(fullcheck) loopi(8) if(!isvalidcube(ch[i])) // not so good...
     {
         emptyfaces(ch[i]);
@@ -417,10 +417,10 @@ static int remipprogress = 0, remiptotal = 0;
 
 bool remip(cube &c, int x, int y, int z, int size)
 {
-    if(c.ext)
+    if(c.merged)
     {
-        c.ext->merged = 0;
-        if(c.ext->merges) freemergeinfo(c);
+        c.merged = 0;
+        if(c.ext && c.ext->merges) freemergeinfo(c);
     }
 
     cube *ch = c.children;
@@ -449,18 +449,18 @@ bool remip(cube &c, int x, int y, int z, int size)
     uchar mat = MAT_AIR;
     loopi(8)
     {
-        mat = ch[i].ext ? ch[i].ext->material : MAT_AIR;
+        mat = ch[i].material;
         if((mat&MATF_CLIP) == MAT_NOCLIP || mat&MAT_ALPHA)
         {
             if(i > 0) return false;
-            while(++i < 8) if(!ch[i].ext || ch[i].ext->material != mat) return false;
+            while(++i < 8) if(ch[i].material != mat) return false;
             break;
         }
         else if(!isentirelysolid(ch[i]))
         {
             while(++i < 8)
             {
-                uchar omat = ch[i].ext ? ch[i].ext->material : MAT_AIR;
+                int omat = ch[i].material;
                 if(isentirelysolid(ch[i]) ? (omat&MATF_CLIP) == MAT_NOCLIP || omat&MAT_ALPHA : mat != omat) return false;
             }
             break;
@@ -502,7 +502,7 @@ bool remip(cube &c, int x, int y, int z, int size)
     freeocta(nh);
     discardchildren(c);
     loopi(3) c.faces[i] = n.faces[i];
-    if(mat!=MAT_AIR) ext(c).material = mat;
+    c.material = mat;
     brightencube(c);
     return true;
 }
@@ -903,13 +903,13 @@ static inline bool occludesface(cube &c, int orient, const ivec &o, int size, co
     int dim = dimension(orient);
     if(!c.children)
     {
-         if(nmat != MAT_AIR && c.ext && (c.ext->material&matmask) == nmat)
+         if(nmat != MAT_AIR && (c.material&matmask) == nmat)
          {
             facevec nf[8];
             return clipfacevecs(vf, o[C[dim]], o[R[dim]], size, nf) < 3;
          }
          if(isentirelysolid(c)) return true;
-         if(vmat != MAT_AIR && c.ext && ((c.ext->material&matmask) == vmat || (isliquid(vmat) && isclipped(c.ext->material&MATF_VOLUME)))) return true;
+         if(vmat != MAT_AIR && ((c.material&matmask) == vmat || (isliquid(vmat) && isclipped(c.material&MATF_VOLUME)))) return true;
          if(touchingface(c, orient) && faceedges(c, orient) == F_SOLID) return true;
          facevec cf[8];
          int numc = clipfacevecs(vf, o[C[dim]], o[R[dim]], size, cf);
@@ -950,9 +950,9 @@ bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat,
 
     if(nsize > size || (nsize == size && !o.children))
     {
-        if(nmat != MAT_AIR && o.ext && (o.ext->material&matmask) == nmat) return true;
+        if(nmat != MAT_AIR && (o.material&matmask) == nmat) return true;
         if(isentirelysolid(o)) return false;
-        if(mat != MAT_AIR && o.ext && ((o.ext->material&matmask) == mat || (isliquid(mat) && (o.ext->material&MATF_VOLUME) == MAT_GLASS))) return false;
+        if(mat != MAT_AIR && ((o.material&matmask) == mat || (isliquid(mat) && (o.material&MATF_VOLUME) == MAT_GLASS))) return false;
         if(isempty(o)) return true;
         if(touchingface(o, opposite(orient)) && faceedges(o, opposite(orient)) == F_SOLID) return false;
 
@@ -995,7 +995,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size)
     cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
     if(&o==&c) return 0;
 
-    uchar nmat = c.ext && c.ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, matmask = MAT_ALPHA;
+    uchar nmat = c.material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, matmask = MAT_ALPHA;
 
     ivec vo(x, y, z);
     vo.mask(0xFFF);
@@ -1005,7 +1005,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size)
     if(nsize > size || (nsize == size && !o.children))
     {
         if(isempty(o)) return 3;
-        if(nmat != MAT_AIR && o.ext && (o.ext->material&matmask) == nmat) return 3;
+        if(nmat != MAT_AIR && (o.material&matmask) == nmat) return 3;
         if(!notouch && (isentirelysolid(o) || (touchingface(o, opp) && faceedges(o, opp) == F_SOLID))) return 0;
 
         genfacevecs(c, orient, vo, size, false, cf);
@@ -1122,7 +1122,7 @@ void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
     p.o.add(p.r);
 
     p.size = 0;
-    p.visible = c.ext ? c.ext->visible : 0;
+    p.visible = c.visible;
     loopi(6) if(usefaces[i] && !touchingface(c, i)) // generate actual clipping planes
     {
         if(flataxisface(c, i)) p.visible |= 1<<i;
@@ -1141,13 +1141,13 @@ void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
     }
 }
 
-static int mergefacecmp(const cubeface *x, const cubeface *y)
+static inline bool mergefacecmp(const cubeface &x, const cubeface &y)
 {
-    if(x->v2 < y->v2) return -1;
-    if(x->v2 > y->v2) return 1;
-    if(x->u1 < y->u1) return -1;
-    if(x->u1 > y->u1) return 1;
-    return 0;
+    if(x.v2 < y.v2) return true;
+    if(x.v2 > y.v2) return false;
+    if(x.u1 < y.u1) return true;
+    if(x.u1 > y.u1) return false;
+    return false;
 }
 
 static int mergefacev(int orient, cubeface *m, int sz, cubeface &n)
@@ -1157,8 +1157,8 @@ static int mergefacev(int orient, cubeface *m, int sz, cubeface &n)
         if(m[i].v2 < n.v1) break;
         if(m[i].v2 == n.v1 && m[i].u1 == n.u1 && m[i].u2 == n.u2)
         {
-            if(m[i].c) ext(*m[i].c).merged |= 1<<orient;
-            if(n.c) ext(*n.c).merged |= 1<<orient;
+            if(m[i].c) m[i].c->merged |= 1<<orient;
+            if(n.c) n.c->merged |= 1<<orient;
             n.v1 = m[i].v1;
             memmove(&m[i], &m[i+1], (sz - (i+1)) * sizeof(cubeface));
             return 1;
@@ -1171,8 +1171,8 @@ static int mergefaceu(int orient, cubeface &m, cubeface &n)
 {
     if(m.v1 == n.v1 && m.v2 == n.v2 && m.u2 == n.u1)
     {
-        if(m.c) ext(*m.c).merged |= 1<<orient;
-        if(n.c) ext(*n.c).merged |= 1<<orient;
+        if(m.c) m.c->merged |= 1<<orient;
+        if(n.c) n.c->merged |= 1<<orient;
         n.u1 = m.u1;
         return 1;
     }
@@ -1247,7 +1247,7 @@ void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo
     uc2 = min(uc2, orig.u2);
     vc1 = max(vc1, orig.v1);
     vc2 = min(vc2, orig.v2);
-    if(!isempty(cu) && touchingface(cu, orient) && !(nmat!=MAT_AIR && cu.ext && (cu.ext->material&matmask)==nmat))
+    if(!isempty(cu) && touchingface(cu, orient) && !(nmat!=MAT_AIR && (cu.material&matmask)==nmat))
     {
         uchar r1 = cu.edges[faceedgesrcidx[orient][0]], r2 = cu.edges[faceedgesrcidx[orient][1]],
               c1 = cu.edges[faceedgesrcidx[orient][2]], c2 = cu.edges[faceedgesrcidx[orient][3]];
@@ -1286,7 +1286,7 @@ bool mincubeface(cube &cu, int orient, const ivec &co, int size, mergeinfo &orig
     mincf.u2 = orig.u1;
     mincf.v1 = orig.v2;
     mincf.v2 = orig.v1;
-    mincubeface(nc, opposite(orient), no, nsize, orig, mincf, cu.ext && cu.ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA);
+    mincubeface(nc, opposite(orient), no, nsize, orig, mincf, cu.material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA);
     bool smaller = false;
     if(mincf.u1 > orig.u1) { orig.u1 = mincf.u1; smaller = true; }
     if(mincf.u2 < orig.u2) { orig.u2 = mincf.u2; smaller = true; }
@@ -1365,7 +1365,7 @@ bool gencubeface(cube &cu, int orient, const ivec &co, int size, ivec &n, int &o
 
     if(minface && touchingface(cu, orient) && mincubeface(cu, orient, co, size, cf))
     {
-        ext(cu).merged |= 1<<orient;
+        cu.merged |= 1<<orient;
     }
 
     return true;
@@ -1374,27 +1374,21 @@ bool gencubeface(cube &cu, int orient, const ivec &co, int size, ivec &n, int &o
 void addmergeinfo(cube &c, int orient, cubeface &cf)
 {
     if(!c.ext) newcubeext(c);
-    int index = 0;
-    loopi(orient) if(c.ext->mergeorigin&(1<<i)) index++;
-    int total = index;
-    loopi(6-orient-1) if(c.ext->mergeorigin&(1<<(i+orient+1))) total++;
-    mergeinfo *m = new mergeinfo[total+1];
-    if(index) memcpy(m, c.ext->merges, index*sizeof(mergeinfo));
-    if(total>index) memcpy(&m[index+1], &c.ext->merges[index], (total-index)*sizeof(mergeinfo));
-    if(c.ext->merges) delete[] c.ext->merges;
-    c.ext->merges = m;
-    m += index;
-    c.ext->mergeorigin |= 1<<orient;
-    m->u1 = cf.u1;
-    m->u2 = cf.u2;
-    m->v1 = cf.v1;
-    m->v2 = cf.v2;
+    if(!c.ext->merges)
+    {
+        c.ext->merges = new mergeinfo[6]; 
+        memset(c.ext->merges, 0, 6*sizeof(mergeinfo));
+    }
+    mergeinfo &m = c.ext->merges[orient];
+    m.u1 = cf.u1;
+    m.u2 = cf.u2;
+    m.v1 = cf.v1;
+    m.v2 = cf.v2;
 }
 
 void freemergeinfo(cube &c)
 {
     if(!c.ext) return;
-    c.ext->mergeorigin = 0;
     DELETEA(c.ext->merges);
 }
 
@@ -1409,13 +1403,13 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
     loopi(8)
     {
         ivec co(i, o.x, o.y, o.z, size);
-        if(c[i].ext)
+        if(c[i].merged)
         {
-            if(c[i].ext->merges) freemergeinfo(c[i]);
-            c[i].ext->merged = 0;
+            c[i].merged = 0;
+            if(c[i].ext && c[i].ext->merges) freemergeinfo(c[i]);
         }
         if(c[i].children) genmergeinfo(c[i].children, co, size>>1);
-        else if(!isempty(c[i])) loopj(6) if(visibleface(c[i], j, co.x, co.y, co.z, size, MAT_AIR, c[i].ext && c[i].ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA))
+        else if(!isempty(c[i])) loopj(6) if(visibleface(c[i], j, co.x, co.y, co.z, size, MAT_AIR, c[i].material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA))
         {
             cfkey k;
             cubeface cf;
@@ -1423,12 +1417,12 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
             {
                 if(size >= 1<<maxmerge || c == worldroot)
                 {
-                    if(c[i].ext && c[i].ext->merged&(1<<j)) addmergeinfo(c[i], j, cf);
+                    if(c[i].merged&(1<<j)) addmergeinfo(c[i], j, cf);
                     continue;
                 }
                 k.orient = j;
                 k.tex = c[i].texture[j];
-                k.material = c[i].ext ? c[i].ext->material&MAT_ALPHA : 0;
+                k.material = c[i].material&MAT_ALPHA;
                 cfaces[k].faces.add(cf);
             }
         }
@@ -1437,7 +1431,7 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
             ASSERT(size <= 1<<maxmerge);
             enumeratekt(cfaces, cfkey, key, cfval, val,
                 val.faces.shrink(mergefaces(key.orient, val.faces.getbuf(), val.faces.length()));
-                loopvj(val.faces) if(val.faces[j].c->ext && val.faces[j].c->ext->merged&(1<<key.orient))
+                loopvj(val.faces) if(val.faces[j].c->merged&(1<<key.orient))
                 {
                     addmergeinfo(*val.faces[j].c, key.orient, val.faces[j]);
                 }
@@ -1508,6 +1502,12 @@ int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, con
 
 static void invalidatemerges(cube &c)
 {
+    if(c.merged)
+    {
+        brightencube(c);
+        c.merged = 0;
+        if(c.ext && c.ext->merges) freemergeinfo(c);
+    }
     if(c.ext)
     {
         if(c.ext->va)
@@ -1516,12 +1516,6 @@ static void invalidatemerges(cube &c)
             destroyva(c.ext->va);
             c.ext->va = NULL;
         }
-        if(c.ext->merged)
-        {
-            brightencube(c);
-            c.ext->merged = 0;
-            if(c.ext->merges) freemergeinfo(c);
-        }
         if(c.ext->tjoints >= 0) c.ext->tjoints = -1;
     }
     if(c.children) loopi(8) invalidatemerges(c.children[i]);
