@@ -31,7 +31,7 @@ void newfont(char *name, char *tex, int *defaultw, int *defaulth, int *offsetx,
 void fontoffset(char *c)
 {
     if(!fontdef) return;
-    
+
     fontdef->charoffset = c[0];
 }
 
@@ -57,6 +57,10 @@ bool setfont(const char *name)
     curfont = f;
     return true;
 }
+ICOMMAND(setfont, "s", (const char *name),
+    if(!setfont(name))
+        conoutf("font %s does not exist", name);
+)
 
 static vector<font *> fontstack;
 
@@ -96,6 +100,7 @@ int text_width(const char *str) { //@TODO deprecate in favour of text_bounds(..)
     text_bounds(str, width, height);
     return width;
 }
+ICOMMAND(text_width, "s", (char *s), intret(text_width(s)))
 
 void tabify(const char *str, int *numtabs)
 {
