@@ -272,7 +272,7 @@ COMMAND(mapsound, "sii");
 void resetchannels()
 {
     loopv(channels) if(channels[i].inuse) freechannel(i);
-    channels.setsize(0);
+    channels.shrink(0);
 }
 
 void clear_sound()
@@ -282,8 +282,8 @@ void clear_sound()
     stopmusic();
     Mix_CloseAudio();
     resetchannels();
-    gamesounds.setsizenodelete(0);
-    mapsounds.setsizenodelete(0);
+    gamesounds.setsize(0);
+    mapsounds.setsize(0);
     samples.clear();
 }
 
@@ -294,7 +294,7 @@ void clearmapsounds()
         Mix_HaltChannel(i);
         freechannel(i);
     }
-    mapsounds.setsizenodelete(0);
+    mapsounds.setsize(0);
 }
 
 COMMANDN(mapsoundreset, clearmapsounds, "");
@@ -319,9 +319,9 @@ void checkmapsounds()
     {
         extentity &e = *ents[i];
         if(e.type!=ET_SOUND) continue;
-        if(camera1->o.dist(e.o) < e.attr2)
+        if(camera1->o.dist(e.o) < e.attr[1])
         {
-            if(!e.visible) playsound(e.attr1, NULL, &e, -1);
+            if(!e.visible) playsound(e.attr[0], NULL, &e, -1);
         }
         else if(e.visible) stopmapsound(&e);
     }
@@ -338,23 +338,23 @@ bool updatechannel(soundchannel &chan)
     if(chan.hasloc())
     {
         vec v;
-        float dist = camera1->o.dist(chan.loc, v);
+        float dist = chan.loc.dist(camera1->o, v);
         int rad = maxsoundradius;
         if(chan.ent)
         {
-            rad = chan.ent->attr2;
-            if(chan.ent->attr3)
+            rad = chan.ent->attr[1];
+            if(chan.ent->attr[2])
             {
-                rad -= chan.ent->attr3;
-                dist -= chan.ent->attr3;
+                rad -= chan.ent->attr[2];
+                dist -= chan.ent->attr[2];
             }
         }
         else if(chan.radius > 0) rad = maxsoundradius ? min(maxsoundradius, chan.radius) : chan.radius;
         if(rad > 0) vol -= int(clamp(dist/rad, 0.0f, 1.0f)*soundvol); // simple mono distance attenuation
         if(stereo && (v.x != 0 || v.y != 0) && dist>0)
         {
-            float yaw = -atan2f(v.x, v.y) - camera1->yaw*RAD; // relative angle of sound along X-Y axis
-            pan = int(255.9f*(0.5f*sinf(yaw)+0.5f)); // range is from 0 (left) to 255 (right)
+            v.rotate_around_z(-camera1->yaw*RAD);
+            pan = int(255.9f*(0.5f - 0.5f*v.x/v.magnitude2())); // range is from 0 (left) to 255 (right)
         }
     }
     vol = (vol*MAXVOL*chan.slot->volume)/255/255;
@@ -416,7 +416,7 @@ static Mix_Chunk *loadwav(const char *name)
     return c;
 }
 
-int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int chanid, int radius)
+int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int chanid, int radius, int expire)
 {
     if(nosound || !soundvol) return -1;
 
@@ -503,9 +503,9 @@ int playsound(int n, const vec *loc, extentity *ent, int loops, int fade, int ch
     if(fade)
     {
         Mix_Volume(chanid, chan.volume);
-        playing = Mix_FadeInChannel(chanid, slot.sample->chunk, loops, fade);
+        playing = expire >= 0 ? Mix_FadeInChannelTimed(chanid, slot.sample->chunk, loops, fade, expire) : Mix_FadeInChannel(chanid, slot.sample->chunk, loops, fade);
     }
-    else playing = Mix_PlayChannel(chanid, slot.sample->chunk, loops);
+    else playing = expire >= 0 ? Mix_PlayChannelTimed(chanid, slot.sample->chunk, loops, expire) : Mix_PlayChannel(chanid, slot.sample->chunk, loops);
     if(playing >= 0) syncchannel(chan);
     else freechannel(chanid);
     SDL_UnlockAudio();
@@ -533,12 +533,12 @@ bool stopsound(int n, int chanid, int fade)
     return true;
 }
 
-int playsoundname(const char *s, const vec *loc, int vol, int loops, int fade, int chanid, int radius)
+int playsoundname(const char *s, const vec *loc, int vol, int loops, int fade, int chanid, int radius, int expire)
 {
     if(!vol) vol = 100;
     int id = findsound(s, vol, gamesounds);
     if(id < 0) id = addsound(s, vol, 0, gamesounds);
-    return playsound(id, loc, NULL, loops, fade, chanid, radius);
+    return playsound(id, loc, NULL, loops, fade, chanid, radius, expire);
 }
 
 void sound(int *n) { playsound(*n); }
@@ -585,8 +585,8 @@ void resetsound()
         DELETEA(musicfile);
         DELETEA(musicdonecmd);
         music = NULL;
-        gamesounds.setsizenodelete(0);
-        mapsounds.setsizenodelete(0);
+        gamesounds.setsize(0);
+        mapsounds.setsize(0);
         samples.clear();
         return;
     }
@@ -688,9 +688,9 @@ void closemumble()
 
 static inline vec mumblevec(const vec &v, bool pos = false)
 {
-    // change from Z up, -Y forward to Y up, +Z forward
+    // change from X left, Z up, Y forward to X right, Y up, Z forward
     // 8 cube units = 1 meter
-    vec m(v.x, v.z, -v.y);
+    vec m(-v.x, v.z, v.y);
     if(pos) m.div(8);
     return m;
 }
