@@ -100,7 +100,7 @@ struct editline
     void del(int start, int count)
     {
         if(!text) return;
-        if(start < 0) { count += start; start = 0; } 
+        if(start < 0) { count += start; start = 0; }
         if(count <= 0 || start >= len) return;
         if(start + count > len) count = len - start - 1;
         memmove(&text[start], &text[start+count], len + 1 - (start + count));
@@ -135,34 +135,34 @@ struct editline
         }
     }
 };
-        
-struct editor 
+
+struct editor
 {
     int mode; //editor mode - 1= keep while focused, 2= keep while used in gui, 3= keep forever (i.e. until mode changes)
-    bool active;
+    bool active, rendered;
     const char *name;
     const char *filename;
 
     int cx, cy; // cursor position - ensured to be valid after a region() or currentline()
     int mx, my; // selection mark, mx=-1 if following cursor - avoid direct access, instead use region()
     int maxx, maxy; // maxy=-1 if unlimited lines, 1 if single line editor
-    
+
     int scrolly; // vertical scroll offset
-    
+
     bool linewrap;
     int pixelwidth; // required for up/down/hit/draw/bounds
     int pixelheight; // -1 for variable sized, i.e. from bounds()
-    
+
     vector<editline> lines; // MUST always contain at least one line!
-        
-    editor(const char *name, int mode, const char *initval) : 
-        mode(mode), active(true), name(newstring(name)), filename(NULL),
+
+    editor(const char *name, int mode, const char *initval) :
+        mode(mode), active(true), rendered(false), name(newstring(name)), filename(NULL),
         cx(0), cy(0), mx(-1), maxx(-1), maxy(-1), scrolly(0), linewrap(false), pixelwidth(-1), pixelheight(-1)
     {
         //printf("editor %08x '%s'\n", this, name);
         lines.add().set(initval ? initval : "");
     }
-    
+
     ~editor()
     {
         //printf("~editor %08x '%s'\n", this, name);
@@ -170,28 +170,28 @@ struct editor
         DELETEA(filename);
         clear(NULL);
     }
-        
+
     void clear(const char *init = "")
     {
         cx = cy = 0;
         mark(false);
         loopv(lines) lines[i].clear();
-        lines.setsize(0);
+        lines.shrink(0);
         if(init) lines.add().set(init);
     }
-    
+
     void setfile(const char *fname)
     {
-        DELETEA(filename); 
+        DELETEA(filename);
         if(fname) filename = newstring(fname);
     }
-    
+
     void load()
     {
         if(!filename) return;
         clear(NULL);
         stream *file = openfile(filename, "r");
-        if(file) 
+        if(file)
         {
             while(lines.add().read(file, maxx) && (maxy < 0 || lines.length() <= maxy));
             lines.pop().clear();
@@ -199,7 +199,7 @@ struct editor
         }
         if(lines.empty()) lines.add().set("");
     }
-    
+
     void save()
     {
         if(!filename) return;
@@ -208,29 +208,29 @@ struct editor
         loopv(lines) file->putline(lines[i].text);
         delete file;
     }
-   
-    void mark(bool enable) 
+
+    void mark(bool enable)
     {
         mx = (enable) ? cx : -1;
         my = cy;
     }
-        
+
     void selectall()
     {
         mx = my = INT_MAX;
         cx = cy = 0;
     }
-    
+
     // constrain results to within buffer - s=start, e=end, return true if a selection range
     // also ensures that cy is always within lines[] and cx is valid
     bool region(int &sx, int &sy, int &ex, int &ey)
     {
-        int n = lines.length(); 
+        int n = lines.length();
         assert(n != 0);
         if(cy < 0) cy = 0; else if(cy >= n) cy = n-1;
         int len = lines[cy].len;
         if(cx < 0) cx = 0; else if(cx > len) cx = len;
-        if(mx >= 0) 
+        if(mx >= 0)
         {
             if(my < 0) my = 0; else if(my >= n) my = n-1;
             len = lines[my].len;
@@ -241,16 +241,16 @@ struct editor
         ex = cx;
         ey = cy;
         if(sy > ey) { swap(sy, ey); swap(sx, ex); }
-        else if(sy==ey && sx > ex) swap(sx, ex);        
+        else if(sy==ey && sx > ex) swap(sx, ex);
         return (sx != ex) || (sy != ey);
     }
-   
+
     bool region() { int sx, sy, ex, ey; return region(sx, sy, ex, ey); }
 
     // also ensures that cy is always within lines[] and cx is valid
     editline &currentline()
     {
-        int n = lines.length(); 
+        int n = lines.length();
         assert(n != 0);
         if(cy < 0) cy = 0; else if(cy >= n) cy = n-1;
         if(cx < 0) cx = 0; else if(cx > lines[cy].len) cx = lines[cy].len;
@@ -328,16 +328,16 @@ struct editor
         loopi(count) lines[start+i].clear();
         lines.remove(start, count);
     }
-            
+
     void del() // removes the current selection (if any)
     {
         int sx, sy, ex, ey;
-        if(!region(sx, sy, ex, ey)) 
-        { 
-            mark(false); 
-            return; 
+        if(!region(sx, sy, ex, ey))
+        {
+            mark(false);
+            return;
         }
-        if(sy == ey) 
+        if(sy == ey)
         {
             if(sx == 0 || ex == lines[ey].len) removelines(sy, 1);
             else lines[sy].del(sx, ex - sx);
@@ -353,12 +353,12 @@ struct editor
         cx = sx;
         cy = sy;
     }
-        
-    void insert(char ch) 
+
+    void insert(char ch)
     {
         del();
         editline &current = currentline();
-        if(ch == '\n') 
+        if(ch == '\n')
         {
             if(maxy == -1 || cy < maxy-1)
             {
@@ -369,7 +369,7 @@ struct editor
             }
             else current.chop(cx);
             cx = 0;
-        } 
+        }
         else
         {
             int len = current.len;
@@ -383,31 +383,31 @@ struct editor
         while(*s) insert(*s++);
     }
 
-    void insertallfrom(editor *b) 
-    {   
+    void insertallfrom(editor *b)
+    {
         if(b==this) return;
 
         del();
-        
-        if(b->lines.length() == 1 || maxy == 1) 
+
+        if(b->lines.length() == 1 || maxy == 1)
         {
             editline &current = currentline();
-            char *str = b->lines[0].text;            
+            char *str = b->lines[0].text;
             int slen = b->lines[0].len;
             if(maxx >= 0 && b->lines[0].len + cx > maxx) slen = maxx-cx;
-            if(slen > 0) 
+            if(slen > 0)
             {
                 int len = current.len;
                 if(maxx >= 0 && slen + cx + len > maxx) len = max(0, maxx-(cx+slen));
-                current.insert(str, cx, slen); 
+                current.insert(str, cx, slen);
                 cx += slen;
             }
-        } 
-        else 
+        }
+        else
         {
-            loopv(b->lines) 
-            {   
-                if(!i) 
+            loopv(b->lines)
+            {
+                if(!i)
                 {
                     lines[cy++].append(b->lines[i].text);
                 }
@@ -423,12 +423,12 @@ struct editor
 
     void key(int code, int cooked)
     {
-        switch(code) 
+        switch(code)
         {
             case SDLK_UP:
-                if(linewrap) 
+                if(linewrap)
                 {
-                    int x, y; 
+                    int x, y;
                     char *str = currentline().text;
                     text_pos(str, cx+1, x, y, pixelwidth);
                     if(y > 0) { cx = text_visible(str, x, y-FONTH, pixelwidth); break; }
@@ -436,7 +436,7 @@ struct editor
                 cy--;
                 break;
             case SDLK_DOWN:
-                if(linewrap) 
+                if(linewrap)
                 {
                     int x, y, width, height;
                     char *str = currentline().text;
@@ -487,7 +487,7 @@ struct editor
             {
                 del();
                 editline &current = currentline();
-                if(cx > 0) current.del(--cx, 1); 
+                if(cx > 0) current.del(--cx, 1);
                 else if(cy > 0)
                 {   //combine with previous line
                     cx = lines[cy-1].len;
@@ -499,7 +499,7 @@ struct editor
             case SDLK_LSHIFT:
             case SDLK_RSHIFT:
                 break;
-            case SDLK_RETURN:    
+            case SDLK_RETURN:
                 cooked = '\n';
                 // fall through
             default:
@@ -517,10 +517,10 @@ struct editor
             int width, height;
             text_bounds(lines[i].text, width, height, maxwidth);
             if(h + height > pixelheight) break;
-            
-            if(hity >= h && hity <= h+height) 
+
+            if(hity >= h && hity <= h+height)
             {
-                int x = text_visible(lines[i].text, hitx, hity-h, maxwidth); 
+                int x = text_visible(lines[i].text, hitx, hity-h, maxwidth);
                 if(dragged) { mx = x; my = i; } else { cx = x; cy = i; };
                 break;
             }
@@ -531,13 +531,13 @@ struct editor
     void draw(int x, int y, int color, bool hit)
     {
         int maxwidth = linewrap?pixelwidth:-1;
-        
+
         int sx, sy, ex, ey;
         bool selection = region(sx, sy, ex, ey);
-        
+
         // fix scrolly so that <cx, cy> is always on screen
         if(cy < scrolly) scrolly = cy;
-        else 
+        else
         {
             if(scrolly < 0) scrolly = 0;
             int h = 0;
@@ -549,7 +549,7 @@ struct editor
                 h += height;
             }
         }
-        
+
         if(selection)
         {
             // convert from cursor coords into pixel coords
@@ -561,15 +561,15 @@ struct editor
             for(int i = scrolly; i < maxy; i++)
             {
                 int width, height;
-                text_bounds(lines[i].text, width, height, maxwidth);                
+                text_bounds(lines[i].text, width, height, maxwidth);
                 if(h + height > pixelheight) { maxy = i; break; }
                 if(i == sy) psy += h;
                 if(i == ey) { pey += h; break; }
                 h += height;
             }
             maxy--;
-            
-            if(ey >= scrolly && sy <= maxy) 
+
+            if(ey >= scrolly && sy <= maxy)
             {
                 // crop top/bottom within window
                 if(sy < scrolly) { sy = scrolly; psy = 0; psx = 0; }
@@ -579,19 +579,19 @@ struct editor
                 glDisable(GL_TEXTURE_2D);
                 glColor3ub(0xA0, 0x80, 0x80);
                 glBegin(GL_QUADS);
-                if(psy == pey) 
+                if(psy == pey)
                 {
                     glVertex2f(x+psx, y+psy);
                     glVertex2f(x+pex, y+psy);
                     glVertex2f(x+pex, y+pey+FONTH);
                     glVertex2f(x+psx, y+pey+FONTH);
-                } 
-                else 
+                }
+                else
                 {   glVertex2f(x+psx,        y+psy);
                     glVertex2f(x+psx,        y+psy+FONTH);
                     glVertex2f(x+pixelwidth, y+psy+FONTH);
                     glVertex2f(x+pixelwidth, y+psy);
-                    if(pey-psy > FONTH) 
+                    if(pey-psy > FONTH)
                     {
                         glVertex2f(x,            y+psy+FONTH);
                         glVertex2f(x+pixelwidth, y+psy+FONTH);
@@ -608,25 +608,25 @@ struct editor
                 defaultshader->set();
             }
         }
-    
+
         int h = 0;
         for(int i = scrolly; i < lines.length(); i++)
         {
             int width, height;
             text_bounds(lines[i].text, width, height, maxwidth);
             if(h + height > pixelheight) break;
-            
+
             draw_text(lines[i].text, x, y+h, color>>16, (color>>8)&0xFF, color&0xFF, 0xFF, hit&&(cy==i)?cx:-1, maxwidth);
             if(linewrap && height > FONTH) // line wrap indicator
-            {   
+            {
                 notextureshader->set();
                 glDisable(GL_TEXTURE_2D);
                 glColor3ub(0x80, 0xA0, 0x80);
-                glBegin(GL_QUADS);
+                glBegin(GL_TRIANGLE_STRIP);
                 glVertex2f(x,         y+h+FONTH);
                 glVertex2f(x,         y+h+height);
-                glVertex2f(x-FONTW/2, y+h+height);
                 glVertex2f(x-FONTW/2, y+h+FONTH);
+                glVertex2f(x-FONTW/2, y+h+height);
                 glEnd();
                 glEnable(GL_TEXTURE_2D);
                 defaultshader->set();
@@ -641,23 +641,23 @@ static vector <editor*> editors;
 
 static editor *currentfocus() { return editors.length() ? editors.last() : NULL; }
 
-static void readyeditors() 
+static void readyeditors()
 {
     loopv(editors) editors[i]->active = (editors[i]->mode==EDITORFOREVER);
 }
 
-static void flusheditors() 
+static void flusheditors()
 {
-    loopvrev(editors) if(!editors[i]->active) 
+    loopvrev(editors) if(!editors[i]->active)
     {
         editor *e = editors.remove(i);
         DELETEP(e);
     }
 }
 
-static editor *useeditor(const char *name, int mode, bool focus, const char *initval = NULL) 
+static editor *useeditor(const char *name, int mode, bool focus, const char *initval = NULL)
 {
-    loopv(editors) if(strcmp(editors[i]->name, name) == 0) 
+    loopv(editors) if(strcmp(editors[i]->name, name) == 0)
     {
         editor *e = editors[i];
         if(focus) { editors.add(e); editors.remove(i); } // re-position as last
@@ -665,7 +665,7 @@ static editor *useeditor(const char *name, int mode, bool focus, const char *ini
         return e;
     }
     editor *e = new editor(name, mode, initval);
-    if(focus) editors.add(e); else editors.insert(0, e); 
+    if(focus) editors.add(e); else editors.insert(0, e);
     return e;
 }
 
@@ -693,7 +693,7 @@ static void removeeditor(editor *e)
 ICOMMAND(textlist, "", (), // @DEBUG return list of all the editors
     string s = "";
     loopv(editors)
-    {   
+    {
         if(i > 0) concatstring(s, ", ");
         concatstring(s, editors[i]->name);
     }
@@ -716,12 +716,12 @@ TEXTCOMMAND(textmode, "i", (int *m), // (1= keep while focused, 2= keep while us
     {
         defformatstring(s)("%d", top->mode);
         result(s);
-    } 
+    }
 );
 TEXTCOMMAND(textsave, "s", (char *file),  // saves the topmost (filename is optional)
-    if(*file) top->setfile(path(file, true)); 
+    if(*file) top->setfile(path(file, true));
     top->save();
-);  
+);
 TEXTCOMMAND(textload, "s", (char *file), // loads into the topmost editor, returns filename if no args
     if(*file)
     {
@@ -730,6 +730,16 @@ TEXTCOMMAND(textload, "s", (char *file), // loads into the topmost editor, retur
     }
     else if(top->filename) result(top->filename);
 );
+TEXTCOMMAND(textinit, "sss", (char *name, char *file, char *initval), // loads into named editor if no file assigned and editor has been rendered
+{
+    editor *e = NULL;
+    loopv(editors) if(!strcmp(editors[i]->name, name)) { e = editors[i]; break; }
+    if(e && e->rendered && !e->filename && *file && (e->lines.empty() || (e->lines.length() == 1 && !strcmp(e->lines[0].text, initval))))
+    {
+        e->setfile(path(file, true));
+        e->load();
+    }
+});
 
 #define PASTEBUFFER "#pastebuffer"
 
@@ -749,3 +759,8 @@ TEXTCOMMAND(textexec, "i", (int *selected), // execute script commands from the
     delete[] script;
 );
 
+TEXTCOMMAND(textalias, "is", (int *selected, char* _alias), // save text into alias
+    char *script = *selected ? top->selectiontostring() : top->tostring();
+    alias(_alias, script);
+    delete[] script;
+);
