@@ -1681,23 +1681,33 @@ namespace UI
 
     void resetcursor()
     {
-        cursorx = cursory = 0.5f;
+        if(editmode || world->children.empty())
+            cursorx = cursory = 0.5f;
     }
 
-    bool movecursor(int dx, int dy)
+    bool movecursor(int &dx, int &dy)
     {
-        if(world->children.empty()) return false;
-        float scale = 500.0f / cursorsensitivity;
-        cursorx = clamp(cursorx+dx*(screen->h/(screen->w*scale)), 0.0f, 1.0f);
-        cursory = clamp(cursory+dy/scale, 0.0f, 1.0f);
-        return true;
+        if(editmode || world->children.length())
+        {
+            float scale = 500.0f / cursorsensitivity;
+            cursorx = clamp(cursorx+dx*(screen->h/(screen->w*scale)), 0.0f, 1.0f);
+            cursory = clamp(cursory+dy/scale, 0.0f, 1.0f);
+            if(editmode)
+            {
+                if(cursorx != 1 && cursorx != 0) dx = 0;
+                if(cursory != 1 && cursory != 0) dy = 0;
+                return false;
+            }
+            return true;
+        }
+        return false;
     }
 
-    bool hascursor(bool targeting)
+    bool hascursor()
     {
         if(world->children.length())
         {
-            if(!targeting) return true;
+            if(mainmenu) return true;
             if(world && world->target(cursorx*world->w, cursory*world->h)) return true;
         }
         return false;
@@ -1705,13 +1715,17 @@ namespace UI
 
     void getcursorpos(float &x, float &y)
     {
-        if(world->children.length()) { x = cursorx; y = cursory; }
-        else x = y = 0.5f;
+        if(editmode || world->children.length())
+        {
+            x = cursorx;
+            y = cursory;
+        }
+        else x = y = .5f;
     }
 
     bool keypress(int code, bool isdown, int cooked)
     {
-        if(!hascursor(true)) return false;
+        if(!hascursor()) return false;
         switch(code)
         {
             case -1:
