@@ -150,7 +150,7 @@ VERTWN(vertln, {
     } \
 }
 
-void rendervertwater(uint subdiv, int xo, int yo, int z, uint size, uchar mat = MAT_WATER)
+void rendervertwater(uint subdiv, int xo, int yo, int z, uint size, uchar mat)
 {
     wx1 = xo;
     wy1 = yo;
@@ -213,7 +213,7 @@ uint calcwatersubdiv(int x, int y, int z, uint size)
     return subdiv;
 }
 
-uint renderwaterlod(int x, int y, int z, uint size, uchar mat = MAT_WATER)
+uint renderwaterlod(int x, int y, int z, uint size, uchar mat)
 {
     if(size <= (uint)(32 << waterlod))
     {
@@ -263,7 +263,7 @@ uint renderwaterlod(int x, int y, int z, uint size, uchar mat = MAT_WATER)
         xtraverts += 4; \
     }
 
-void renderflatwater(int x, int y, int z, uint rsize, uint csize, uchar mat = MAT_WATER)
+void renderflatwater(int x, int y, int z, uint rsize, uint csize, uchar mat)
 {
     switch(mat)
     {
@@ -293,17 +293,19 @@ void renderflatwater(int x, int y, int z, uint rsize, uint csize, uchar mat = MA
 
 VARFP(vertwater, 0, 1, 1, allchanged());
 
+static inline void renderwater(const materialsurface &m, int mat = MAT_WATER)
+{
+    if(!vertwater || minimapping) renderflatwater(m.o.x, m.o.y, m.o.z, m.rsize, m.csize, mat);
+    else if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize, mat) >= (uint)m.csize * 2)
+        rendervertwater(m.csize, m.o.x, m.o.y, m.o.z, m.csize, mat);
+}
+
 void renderlava(const materialsurface &m, Texture *tex, float scale)
 {
     lavaxk = 8.0f/(tex->xs*scale);
     lavayk = 8.0f/(tex->ys*scale);
     lavascroll = lastmillis/1000.0f;
-    if(vertwater)
-    {
-        if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize, MAT_LAVA) >= (uint)m.csize * 2)
-            rendervertwater(m.csize, m.o.x, m.o.y, m.o.z, m.csize, MAT_LAVA);
-    }
-    else renderflatwater(m.o.x, m.o.y, m.o.z, m.rsize, m.csize, MAT_LAVA);
+    renderwater(m, MAT_LAVA);
 }
 
 /* reflective/refractive water */
@@ -512,9 +514,7 @@ void renderwaterff()
                 lastdepth = m.depth;
             }
 
-            if(!vertwater) renderflatwater(m.o.x, m.o.y, m.o.z, m.rsize, m.csize);
-            else if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize) >= (uint)m.csize * 2)
-                rendervertwater(m.csize, m.o.x, m.o.y, m.o.z, m.csize);
+            renderwater(m);
         }
         if(varray::data.length()) varray::end();
     }
@@ -728,9 +728,7 @@ void renderwater()
                 lastdepth = m.depth;
             }
 
-            if(!vertwater || minimapping) renderflatwater(m.o.x, m.o.y, m.o.z, m.rsize, m.csize);
-            else if(renderwaterlod(m.o.x, m.o.y, m.o.z, m.csize) >= (uint)m.csize * 2)
-                rendervertwater(m.csize, m.o.x, m.o.y, m.o.z, m.csize);
+            renderwater(m);
         }
         if(varray::data.length()) varray::end();
     }
@@ -1066,6 +1064,8 @@ void queryreflections()
         glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
         glEnable(GL_CULL_FACE);
     }
+
+	glFlush();
 }
 
 VARP(maxreflect, 1, 1, 8);
@@ -1157,7 +1157,7 @@ static bool calcscissorbox(Reflection &ref, int size, vec &clipmin, vec &clipmax
     {
         vec4 &p = v[i];
         mvpmatrix.transform(vec(i&1 ? bbmax.x : bbmin.x, i&2 ? bbmax.y : bbmin.y, (i&4 ? bbmax.z + WATER_AMPLITUDE : bbmin.z - WATER_AMPLITUDE) - WATER_OFFSET), p);
-        if(p.z >= 0)
+        if(p.z >= -p.w)
         {
             float x = p.x / p.w, y = p.y / p.w;
             sx1 = min(sx1, x);
@@ -1170,12 +1170,12 @@ static bool calcscissorbox(Reflection &ref, int size, vec &clipmin, vec &clipmax
     loopi(8)
     {
         const vec4 &p = v[i];
-        if(p.z >= 0) continue;
+        if(p.z >= -p.w) continue;
         loopj(3)
         {
             const vec4 &o = v[i^(1<<j)];
-            if(o.z <= 0) continue;
-            float t = p.z/(p.z - o.z),
+            if(o.z <= -o.w) continue;
+            float t = (p.z + p.w)/(p.z + p.w - o.z - o.w),
                   w = p.w + t*(o.w - p.w),
                   x = (p.x + t*(o.x - p.x))/w,
                   y = (p.y + t*(o.y - p.y))/w;
