@@ -4,6 +4,30 @@ namespace entities
 {
     using namespace game;
 
+    int extraentinfosize() { return 0; }       // size in bytes of what the 2 methods below read/write... so it can be skipped by other games
+
+    void writeent(entity &e, char *buf)   // write any additional data to disk (except for ET_ ents)
+    {
+    }
+
+    void readent(entity &e, char *buf, int ver)     // read from disk, and init
+    {
+        if(ver <= 30) switch(e.type)
+        {
+            case FLAG:
+            case MONSTER:
+            case TELEDEST:
+            case RESPAWNPOINT:
+            case BOX:
+            case BARREL:
+            case PLATFORM:
+            case ELEVATOR:
+                e.attr[0] = (int(e.attr[0])+180)%360;
+                break;
+        }
+    }
+
+#ifndef STANDALONE
     vector<extentity *> ents;
 
     vector<extentity *> &getents() { return ents; }
@@ -36,9 +60,9 @@ namespace entities
             "ammo/shells", "ammo/bullets", "ammo/rockets", "ammo/rrounds", "ammo/grenades", "ammo/cartridges",
             "health", "boost", "armor/green", "armor/yellow", "quad", "teleporter",
             NULL, NULL,
-            "carrot",
+            NULL, //"carrot",
             NULL, NULL,
-            "checkpoint",
+            NULL, // "checkpoint",
             NULL, NULL,
             NULL, NULL,
             NULL, NULL
@@ -806,30 +830,6 @@ namespace entities
 	}
     }
 
-    int extraentinfosize() { return 0; }       // size in bytes of what the 2 methods below read/write... so it can be skipped by other games
-
-    void writeent(entity &e, char *buf)   // write any additional data to disk (except for ET_ ents)
-    {
-    }
-
-    void readent(entity &e, char *buf)     // read from disk, and init
-    {
-        int ver = getmapversion();
-        if(ver <= 30) switch(e.type)
-        {
-            case FLAG:
-            case MONSTER:
-            case TELEDEST:
-            case RESPAWNPOINT:
-            case BOX:
-            case BARREL:
-            case PLATFORM:
-            case ELEVATOR:
-                e.attr[0] = (int(e.attr[0])+180)%360;
-                break;
-        }
-    }
-
     void editent(int i, bool local)
     {
         extentity &e = *ents[i];
@@ -850,36 +850,37 @@ namespace entities
         return 4.0f;
     }
 
-	int *getmodelattr(extentity &e)
-	{
-		switch(e.type)
-		{
-			case TELEPORT:
-			case PLATFORM:
-			case ELEVATOR:
-			case BARREL:
-			case BOX:
-				return &e.attr[1];
-			default:
-				return NULL;
-		}
-	}
+    int *getmodelattr(extentity &e)
+    {
+        switch(e.type)
+        {
+            case TELEPORT:
+            case PLATFORM:
+            case ELEVATOR:
+            case BARREL:
+            case BOX:
+                return &e.attr[1];
+            default:
+                return NULL;
+        }
+    }
 
-	bool checkmodelusage(extentity &e, int i)
-	{
-		switch(e.type)
-		{
-			case TELEPORT:
-				if(e.attr[1] == 0)
-					return false;
-			case PLATFORM:
-			case ELEVATOR:
-			case BARREL:
-			case BOX:
-				return (e.attr[1] == i);
-			default:
-				return false;
-		}
-	}
+    bool checkmodelusage(extentity &e, int i)
+    {
+        switch(e.type)
+        {
+            case TELEPORT:
+                if(e.attr[1] == 0)
+                    return false;
+            case PLATFORM:
+            case ELEVATOR:
+            case BARREL:
+            case BOX:
+                return (e.attr[1] == i);
+            default:
+                return false;
+        }
+    }
+#endif
 }
 
