@@ -7,6 +7,8 @@ namespace game
     int maptime = 0, maprealtime = 0, maplimit = -1;
     int respawnent = -1;
     int lasthit = 0, lastspawnattempt = 0;
+    int moveentropy = 0;
+    VARP(mouselook, 0, 1, 1);
 
     int following = -1, followdir = 0;
 
@@ -305,6 +307,14 @@ namespace game
         else if(!intermission)
         {
             if(player1->ragdoll) cleanragdoll(player1);
+            if(!mouselook)
+            {
+                player1->move = clamp(1, -1, moveentropy);
+                if(moveentropy < 0)
+                    moveentropy += min(-moveentropy, curtime);
+                else
+                    moveentropy -= min(moveentropy, curtime);
+            }
             moveplayer(player1, 10, true);
             swayhudgun(curtime);
             entities::checkitems(player1);
@@ -375,7 +385,7 @@ namespace game
 
     void damaged(int damage, fpsent *d, fpsent *actor, bool local)
     {
-        if(d->state!=CS_ALIVE || intermission) return;
+        if((d->state!=CS_ALIVE && d->state != CS_LAGGED && d->state != CS_SPAWNING) || intermission) return;
 
         if(local) damage = d->dodamage(damage);
         else if(actor==player1) return;
@@ -439,7 +449,7 @@ namespace game
             else d->resetinterp();
             return;
         }
-        else if(d->state!=CS_ALIVE || intermission) return;
+        else if((d->state!=CS_ALIVE && d->state != CS_LAGGED && d->state != CS_SPAWNING) || intermission) return;
 
         fpsent *h = followingplayer();
         if(!h) h = player1;
@@ -900,10 +910,10 @@ namespace game
     {
         switch(index)
         {
-	    case 3: return "data/edit.png";
-            case 2: return "data/hit.png";
-            case 1: return "data/teammate.png";
-            default: return "data/crosshair.png";
+	    case 3: return "packages/crosshairs/edit.png";
+	    case 2: return "packages/crosshairs/hit.png";
+	    case 1: return "packages/crosshairs/teammate.png";
+	    default: return "packages/crosshairs/default.png";
         }
     }
 
@@ -1084,7 +1094,18 @@ namespace game
 
     bool allowdoublejump() {return false;}
     void updatecamera() {}
-    bool mousemove(int &dx, int &dy, float &cursens) {return false;}
+    bool mousemove(int &dx, int &dy, float &cursens)
+    {
+        if(!mouselook)
+        {
+            //remember, vertical coordinates are flipped
+            moveentropy -= dy * cursens;
+            moveentropy = clamp(500, -500, moveentropy);
+            dy = 0;
+        }
+        //we return false since we don't want to do yaw manipulation here
+        return false;
+    }
     bool recomputecamera(physent *&camera1, physent &tempcamera, bool &detachedcamera, float &thirdpersondistance) {return false;}
     void writemapdata(stream *f) {}
 
@@ -1092,7 +1113,6 @@ namespace game
     void writegamedata(vector<char> &extras) {}
     void readgamedata(vector<char> &extras) {}
 
-    const char *gameident() { return "fps"; }
     const char *autoexec() { return "autoexec.cfg"; }
     const char *savedservers() { return "data/fps/servers.cfg"; }
     void loadconfigs()
