@@ -8,22 +8,34 @@ namespace game
 	// GAME DEFINITIONS
 	vector<script *> scripts; ///scripts, includes dialogue
 	vector<effect *> effects; ///pretty particle effects for spells and stuff
-	vector<statusgroup *> statuses; ///TODO find a better name - status effect definitions to transfer onto victims
+	vector<statusgroup *> statuses; ///status effect definitions to transfer onto victims
 	vector<item_base *> items;
 	vector<ammotype *> ammotypes;
 	vector<char_base *> chars;
 	vector<faction *> factions;
-	vector<object_base *> objects;
+	vector<obstacle_base *> obstacles;
+	vector<container_base *> containers;
+	vector<platform_base *> platforms;
+	vector<trigger_base *> triggers;
 	vector<mapscript *> mapscripts;
 	vector<recipe *> recipes;
 
 	vector<int> variables; //global variables
 	vector<const char *> tips;
 
+	string data; //data folder
 	hashtable<const char *, mapinfo> *mapdata = NULL;
 	mapinfo *curmap = NULL;
 	bool connected = false;
 	bool transfer = false;
+	bool abort = false;
+
+	//important variables/configuration
+	SVAR(firstmap, "");
+	VAR(gameversion, 0, 0, 0x7FFFFFFF);
+	VAR(compatversion, 0, 0, 0x7FFFFFFF);
+	//friendlyfire
+	//
 
 	ICOMMAND(primaryattack, "D", (int *down),
 		player1->primary = *down != 0 && ! player1->secondary;
@@ -42,100 +54,11 @@ namespace game
 		*/
 	)
 
-	void cleangame()
-	{
-		rpgscript::clean();
-
-		if(mapdata)
-		{
-			if(DEBUG_WORLD)
-				conoutf(CON_DEBUG, "DEBUG: clearing map data: %p %p", mapdata, curmap);
-
-			if(curmap)
-				curmap->objs.removeobj(player1);
-
-			delete mapdata; mapdata = NULL;
-			curmap = NULL;
-		}
-
-		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: clearing %i scripts, %i effects, %i status effects, %i items, %i ammotypes, %i characters, %i factions, %i objects, %i mapscripts, %i recipes, %i variables and %i tips",
-				scripts.length(), effects.length(), statuses.length(), items.length(), ammotypes.length(), chars.length(), factions.length(), objects.length(), mapscripts.length(), recipes.length(), variables.length(), tips.length());
-
-		scripts.deletecontents();
-		effects.deletecontents();
-		statuses.deletecontents();
-		items.deletecontents();
-		ammotypes.deletecontents();
-		chars.deletecontents();
-		factions.deletecontents();
-		objects.deletecontents();
-		mapscripts.deletecontents();
-		recipes.deletecontents();
-		variables.setsize(0);
-		tips.deletearrays();
-		camera::cleanup(true);
-
-		player1->inventory.deletecontents();
-		player1->equipped.setsize(0);
-	}
-
-	void newgame(const char *game)
-	{
-		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: starting new game");
-
-		cleangame();
-
-		mapdata = new hashtable<const char *, mapinfo>;
-
-		emptymap(0, true, NULL, false);
-
-		defformatstring(file)("data/rpg/games/%s.cfg", game);
-		if(DEBUG_WORLD)
-			conoutf(CON_DEBUG, "DEBUG: attempting to load game definitions from %s", file);
-
-		if(!execfile(file, false))
-		{
-			conoutf(CON_ERROR, "ERROR: data for game \"%s\" does not exist, falling back to \"default\"", game);
-			if(!execfile("data/rpg/games/default.cfg"))
-				fatal("Unable to start game");
-		}
-
-		if(chars.inrange(0))
-		{
-			chars[0]->transfer(*player1);
-			player1->resetmdl();
-			rpgscript::doentscript(player1, player1, SCR_SPAWN);
-		}
-		else
-		{
-			conoutf(CON_ERROR, "ERROR! no character definition for player! continuing with current stats");
-		}
-	}
-	COMMAND(newgame, "s");
-
-	void initclient()
-	{
-	}
-
-	void spawnplayer(rpgent *d)
-	{
-		findplayerspawn(d, -1);
-	}
-
-	void respawnself(bool death = false)
-	{
-		player1->respawn();
-	}
-
-	void respawn()
-	{
-		if(player1->state==CS_DEAD)
-		{
-			respawnself(true);
-		}
-	}
+	ICOMMAND(include, "s", (const char *pth),
+		static string file;
+		formatstring(file)("data/rpg/games/%s/%s.cfg", data, pth);
+		execfile(file);
+	)
 
 	struct collision
 	{
@@ -164,6 +87,224 @@ namespace game
 		collision::add(d, o, dir);
 	}
 
+	void cleangame()
+	{
+		gameversion = compatversion = 0;
+
+		rpgscript::clean();
+		colcache.setsize(0);
+
+		if(mapdata)
+		{
+			if(DEBUG_WORLD)
+				conoutf(CON_DEBUG, "DEBUG: clearing map data: %p %p", mapdata, curmap);
+
+			if(curmap)
+				curmap->objs.removeobj(player1);
+
+			delete mapdata; mapdata = NULL;
+			curmap = NULL;
+		}
+
+		if(DEBUG_WORLD)
+			conoutf(CON_DEBUG, "DEBUG: clearing %i scripts, %i effects, %i status effects, %i items,"
+				" %i ammotypes, %i characters, %i factions, %i obstacles, %i containers, %i platforms,"
+				" %i triggers, %i mapscripts, %i recipes, %i variables and %i tips",
+				scripts.length(), effects.length(), statuses.length(), items.length(),
+				ammotypes.length(), chars.length(), factions.length(), obstacles.length(), containers.length(), platforms.length(),
+				triggers.length(), mapscripts.length(), recipes.length(), variables.length(), tips.length());
+
+		scripts.deletecontents();
+		effects.deletecontents();
+		statuses.deletecontents();
+		items.deletecontents();
+		ammotypes.deletecontents();
+		chars.deletecontents();
+		factions.deletecontents();
+		obstacles.deletecontents();
+		containers.deletecontents();
+		platforms.deletecontents();
+		triggers.deletecontents();
+		mapscripts.deletecontents();
+		recipes.deletecontents();
+		variables.setsize(0);
+		tips.deletearrays();
+		camera::cleanup(true);
+
+		char_base tmp;
+		tmp.transfer(*player1);
+		player1->equipped.setsize(0);
+	}
+
+	template<class T>
+	static inline void loadassets(const char *dir, T *&var, vector<T*> &objects)
+	{
+		objects.deletecontents();
+		vector<char *> files;
+		string file;
+
+		listfiles(dir, "cfg", files);
+		loopv(files)
+		{
+			char *end;
+			int idx = strtol(files[i], &end, 10);
+			if(end == files[i] || *end != '\0' || idx < 0)
+			{
+				conoutf(CON_ERROR, "ERROR: can't load %s - invalid index or file name; skipping", files[i]);
+				continue;
+			}
+			if(DEBUG_WORLD)
+				conoutf(CON_DEBUG, "DEBUG: idx = %i, vector is %i; expanding if necessary", idx, objects.length());
+			while(!objects.inrange(idx)) objects.add(NULL);
+
+			if(objects[idx])
+			{
+				conoutf(CON_ERROR, "ERROR: object with index %i already defined in %s; skipping", idx, dir);
+				continue;
+			}
+
+			objects[idx] = var = new T();
+
+			formatstring(file)("%s/%s.cfg", dir, files[i]);
+			if(DEBUG_WORLD) conoutf("DEBUG: executing %s for index %i", file, idx);
+			execfile(file);
+		}
+		files.deletearrays();
+		var = NULL;
+
+		loopv(objects)
+		{
+			if(!objects[i])
+			{
+				conoutf(CON_ERROR, "ERROR: index %i not defined in %s; problems WILL occur if referenced.", i, dir);
+				abort = true;
+			}
+		}
+	}
+
+	void loadassets(const char *dir, bool definitions = false)
+	{
+		string pth;
+
+		if(!definitions)
+		{
+			formatstring(pth)("%s/variables.cfg", dir);
+			execfile(pth);
+			formatstring(pth)("%s/tips.cfg", dir);
+			execfile(pth);
+		}
+
+		formatstring(pth)("%s/scripts", dir);
+		loadassets(pth, loadingscript, scripts);
+		formatstring(pth)("%s/effects", dir);
+		loadassets(pth, loadingeffect, effects);
+		formatstring(pth)("%s/statuses", dir);
+		loadassets(pth, loadingstatusgroup, statuses);
+		formatstring(pth)("%s/items", dir);
+		loadassets(pth, loadingitem_base, items);
+		formatstring(pth)("%s/ammo", dir);
+		loadassets(pth, loadingammotype, ammotypes);
+		formatstring(pth)("%s/critters", dir);
+		loadassets(pth, loadingchar_base, chars);
+		formatstring(pth)("%s/factions", dir);
+		loadassets(pth, loadingfaction, factions);
+		formatstring(pth)("%s/obstacles", dir);
+		loadassets(pth, loadingobstacle_base, obstacles);
+		formatstring(pth)("%s/containers", dir);
+		loadassets(pth, loadingcontainer_base, containers);
+		formatstring(pth)("%s/platforms", dir);
+		loadassets(pth, loadingplatform_base, platforms);
+		formatstring(pth)("%s/triggers", dir);
+		loadassets(pth, loadingtrigger_base, triggers);
+		formatstring(pth)("%s/mapscripts", dir);
+		loadassets(pth, loadingmapscript, mapscripts);
+		formatstring(pth)("%s/recipes", dir);
+		loadassets(pth, loadingrecipe, recipes);
+
+		if(!definitions)
+		{
+			loadingchar_base = player1;
+			formatstring(pth)("%s/player.cfg", dir);
+			execfile(pth);
+			loadingchar_base = NULL;
+		}
+	}
+
+	ICOMMAND(r_rehash, "", (),
+		if(!mapdata) return;
+		conoutf("Reloading configuration files in data/rpg/games/%s", data);
+		defformatstring(dir)("data/rpg/games/%s", data);
+		loadassets(dir, true);
+		if(abort)
+		{
+			conoutf(CON_ERROR, "ERROR: game breaking errors were encountered while parsing files; aborting current game");
+			abort = false;
+			localdisconnect();
+		}
+	)
+
+	void newgame(const char *game, bool restore)
+	{
+		if(!game || !*game) game = "default";
+		if(DEBUG_WORLD)
+			conoutf(CON_DEBUG, "DEBUG: starting new game: %s", game);
+
+		cleangame();
+		rpgscript::init();
+
+		mapdata = new hashtable<const char *, mapinfo>;
+		emptymap(0, true, NULL, false);
+		copystring(data, game);
+
+		defformatstring(dir)("data/rpg/games/%s", game);
+		{
+			defformatstring(pth)("%s/variables.cfg", dir);
+			execfile(pth);
+			formatstring(pth)("%s/tips.cfg", dir);
+			execfile(pth);
+		}
+		loadassets(dir);
+		if(abort)
+		{
+			abort = false; localdisconnect(); return;
+		}
+		concatstring(dir, ".cfg");
+		if(!execfile(dir))
+			conoutf(CON_WARN, "WARNING: game properties not defined");
+		else if(DEBUG_WORLD)
+			conoutf(CON_DEBUG, "DEBUG: Loaded game properties");
+
+		player1->resetmdl();
+		if(restore) return;
+
+		player1->health = player1->base.getmaxhp();
+		player1->mana = player1->base.getmaxmp();
+		player1->getsignal("spawn", false);
+	}
+	ICOMMAND(newgame, "s", (const char *s), newgame(s));
+
+	void initclient()
+	{
+	}
+
+	void spawnplayer(rpgent *d)
+	{
+		findplayerspawn(d, -1);
+	}
+
+	void respawnself(bool death = false)
+	{
+		player1->respawn();
+	}
+
+	void respawn()
+	{
+		if(player1->state==CS_DEAD)
+		{
+			respawnself(true);
+		}
+	}
+
 	mapinfo *accessmap(const char *name)
 	{
 		if(!mapdata)
@@ -188,11 +329,27 @@ namespace game
 
 	int lasttip = -1;
 
+	void clearmap(mapinfo *map)
+	{
+		colcache.setsize(0);
+		loopvrev(map->objs)
+		{
+			rpgent *ent = map->objs[i];
+			rpgscript::removereferences(ent);
+			delete ent;
+		}
+
+		map->objs.deletecontents();
+		map->projs.deletecontents();
+		map->aeffects.deletecontents();
+		map->blips.setsize(0);
+	}
+
 	void startmap(const char *name)
 	{
 		entities::genentlist();
 		ai::clearwaypoints();
-		camera::cleanup();
+		camera::cleanup(false);
 		if(!name)
 			return;
 
@@ -203,9 +360,17 @@ namespace game
 			if(DEBUG_WORLD)
 				conoutf(CON_DEBUG, "DEBUG: removing player from curmap vector");
 			curmap->objs.removeobj(player1);
+			if(curmap->flags & mapinfo::F_VOLATILE)
+			{
+				if(DEBUG_WORLD)
+					conoutf(CON_DEBUG, "DEBUG: curmap volatile, clearing before map  change");
+				clearmap(curmap);
+				curmap->loaded = false;
+			}
 		}
 
 		curmap = accessmap(name);
+		rpgscript::changemap();
 
 		if(DEBUG_WORLD)
 			conoutf(CON_DEBUG, "DEBUG: adding player to curmap vector");
@@ -216,7 +381,8 @@ namespace game
 			if(DEBUG_WORLD)
 				conoutf(CON_DEBUG, "DEBUG: map not loaded - initialising");
 
-			rpgscript::domapscript(curmap, NULL, MSCR_LOAD);
+			entities::startmap();
+			curmap->getsignal("load", false);
 			curmap->loaded = true;
 		}
 
@@ -247,11 +413,44 @@ namespace game
 			conoutf("\f2%s", tips[lasttip]);
 	}
 
+	void openworld(const char *name, bool fall)
+	{
+		if(!mapdata)
+		{
+			if(DEBUG_WORLD) conoutf(CON_DEBUG, "DEBUG: no game in progress, falling back to default");
+			newgame(NULL);
+		}
+		if(!connected) localconnect();
+
+		if(name && *name && load_world(name))
+			return;
+		else if(fall && load_world(DEFAULTMAP))
+			return;
+		else
+			emptymap(10, true, "untitled");
+	}
+
+	ICOMMAND(map, "s", (char *s), openworld(s, false))
+
+	void changemap(const char *name) { openworld(name, true); }
+	void forceedit(const char *name) { openworld(name, false); }
+
 	void updateworld()
 	{
 		rpggui::forcegui();
-		if(!curtime || !curmap) return;
+		if(!mapdata || !curtime) return;
+		if(!curmap)
+		{
+			openworld(firstmap);
+			return;
+		}
+
 		physicsframe();
+		if(editmode)
+		{
+			moveplayer(player1, 10, false);
+			return;
+		}
 
 		loopv(curmap->objs)
 		{
@@ -268,11 +467,10 @@ namespace game
 					j--;
 				}
 			}
-			if(d->temp.mdl)
-				setbbfrommodel(d, d->temp.mdl);
+			setbbfrommodel(d, d->temp.mdl);
 
 			eye = d->eyeheight - eye;
-			if(eye > 0)
+			if(eye)
 			{
 				if(DEBUG_ENT)
 					conoutf("DEBUG: ent has new eyeheight of %f, applying positional delta of %f", d->eyeheight, eye);
@@ -286,42 +484,46 @@ namespace game
 			//note dir is the direction d is from o based on the collision point
 
 			collision &c = colcache[i];
-			rpgscript::doentscript(c.o, c.d, SCR_COLLIDE);
-			rpgscript::doentscript(c.d, c.o, SCR_COLLIDE);
+			c.o->getsignal("collide", false, c.d);
+			c.d->getsignal("collide", false, c.o);
 
-			if(c.dir.z > 0)
+			//first parse platforms - no discrimination
+			rpgplatform *lower = (rpgplatform *) (c.dir.z > 0 ? c.o : c.d);
+			rpgent *upper = lower == c.o ? c.d : c.o;
+
+			if(c.dir.z && lower->type() == ENT_PLATFORM && upper->type() != ENT_PLATFORM)
+				lower->stack.add(upper);
+
+			//now do pushing - limit to obstacles, NPCs and items
+			rpgent *first = c.o, *second = c.d;
+
+			if((first->type() == ENT_CHAR || first->type() == ENT_ITEM || first->type() == ENT_OBSTACLE) &&
+				(second->type() == ENT_CHAR || second->type() == ENT_ITEM || second->type() == ENT_OBSTACLE))
 			{
-				rpgobject *o = (rpgobject *) c.o;
-				if(o->type() == ENT_OBJECT && o->flags & (rpgobject::MOVABLE|rpgobject::PLATFORM) &&
-					(c.d->type() != ENT_OBJECT || ((rpgobject *) c.d)->flags & rpgobject::MOVABLE))
-					o->stack.add(c.d);
-			}
-			else if (c.dir.z < 0)
-			{
-				rpgobject *d = (rpgobject *) c.d;
-				if(d->type() == ENT_OBJECT && d->flags & (rpgobject::MOVABLE|rpgobject::PLATFORM) &&
-					(c.o->type() != ENT_OBJECT || ((rpgobject *) c.o)->flags & rpgobject::MOVABLE))
-					d->stack.add(c.o);
-			}
-			else
-			{
-				rpgchar *pusher = NULL;
-				rpgobject *pushee = NULL;
+				vec vel1, vel2;
 
-				if(c.o->type() == ENT_CHAR) pusher = (rpgchar *) c.o;
-				else if(c.o->type() == ENT_OBJECT) pushee = (rpgobject *) c.o;
+				vecfromyawpitch(second->yaw, 0, second->move, second->strafe, vel1);
+				vecfromyawpitch(first->yaw, 0, first->move, first->strafe, vel2);
 
-				if(c.d->type() == ENT_CHAR) { pusher = (rpgchar *) c.d; c.dir.neg(); }
-				else if(c.d->type() == ENT_OBJECT) pushee = (rpgobject *) c.d;
+				vel1.mul(second->maxspeed * curtime / 1000.f).add(first->vel);
+				vel2.mul(first->maxspeed * curtime / 1000.f).add(second->vel);
 
-				if(pusher && pushee && pushee->flags & rpgobject::MOVABLE)
-					pushee->vel.add(vec(c.dir).mul(pusher->maxspeed * curtime / 500.0f)); // TODO factor strength and weight into this
+				first->vel = vel1;
+				second->vel = vel2;
+
+
+// 				//TODO factor strength, weight and direction into this
+//
+// 				vel1.add(vec(first->vel).add(first->falling).mul(.2));;
+// 				vel2.add(vec(second->vel).add(second->falling).mul(.2));
+
+				//first->vel.add(vel2);
+				//second->vel.add(vel1);
 			}
 		}
 		colcache.setsize(0);
 
 		ai::trydrop();
-		rpgscript::update();
 
 		loopvrev(curmap->projs)
 		{
@@ -341,6 +543,8 @@ namespace game
 		}
 
 		camera::update();
+		curmap->getsignal("update");
+		rpgscript::update();
 	}
 
 	void setwindowcaption()
@@ -484,41 +688,17 @@ namespace game
 	void newmap(int size)
 	{
 		if(size < 0) return;
-
-		if(!mapdata) newgame("default");
+		if(!mapdata) newgame(NULL);
+		if(!connected) localconnect();
 		emptymap(size, true);
-		if(!connected) localconnect();
 	}
-	ICOMMAND(newmap, "i", (int *size), newmap(max(0, *size));) ///overrides built in variant - we want to reset the game state before creating entities
+	ICOMMAND(newmap, "i", (int *size), newmap(max(*size, 0));) ///overrides built in variant - we want to reset the game state before creating entities
 
-	void gameconnect(bool _remote) {if(!mapdata) newgame("default"); connected = true;}
-
-	void openworld(const char *name, bool fall = false)
-	{
-		if(!mapdata)
-		{
-			if(DEBUG_WORLD) conoutf(CON_DEBUG, "DEBUG: no game in progress, falling back to default");
-			newgame("default");
-		}
-		if(!connected) localconnect();
-
-		if(name && *name && load_world(name))
-			return;
-		else if(fall && load_world(DEFAULTMAP))
-			return;
-		else
-			emptymap(10, true, "untitled");
-	}
-
-	ICOMMAND(map, "s", (char *s), openworld(s, false))
-
-	void changemap(const char *name) { openworld(name, true); }
-	void forceedit(const char *name) { openworld(name, false); }
+	void gameconnect(bool _remote) {if(!mapdata) newgame(NULL); connected = true;}
 
 	void gamedisconnect(bool cleanup)
 	{
 		cleangame();
-
 		connected = false;
 	}
 
@@ -529,11 +709,8 @@ namespace game
 
 	bool cansave()
 	{
-		if(camera::cutscene || rpggui::open())
-		{
-			conoutf("You may not save at this time");
+		if(curmap->flags & mapinfo::F_NOSAVE || camera::cutscene || rpggui::open())
 			return false;
-		}
 		return true;
 	}
 
@@ -550,14 +727,10 @@ namespace game
 					conoutf(CON_DEBUG, "DEBUG: resetting game state for current map");
 
 				curmap->objs.removeobj(player1);
-
-				curmap->objs.deletecontents();
-				curmap->projs.deletecontents();
-				curmap->aeffects.deletecontents();
-				curmap->blips.setsize(0);
-
+				clearmap(curmap);
 				curmap->objs.add(player1);
-				rpgscript::domapscript(curmap, NULL, MSCR_LOAD);
+				entities::startmap();
+				curmap->getsignal("load");
 			}
 
 			entities::genentlist();
@@ -580,7 +753,7 @@ namespace game
 	{
 		if(curmap)
 		{
-			if(i <curmap->objs.length())
+			if(i < curmap->objs.length())
 				return curmap->objs[i];
 		}
 		else
@@ -613,5 +786,5 @@ namespace game
 	void mapfailed(const char *name) {}
 	bool allowdoublejump() { return false; }
 	bool detachcamera() { return player1->state == CS_DEAD; }
-
+	void toserver(char *text) { execute(text); } //since we don't tlak, just execute if the / is forgotten
 }
