@@ -140,6 +140,11 @@ extern void conoutf(const char *s, ...);
 extern void conoutf(int type, const char *s, ...);
 extern void conoutfv(int type, const char *fmt, va_list args);
 
+extern void setlogfile(const char *fname);
+extern void closelogfile();
+extern void logoutfv(const char *fmt, va_list args);
+extern void logoutf(const char *fmt, ...);
+
 // menus
 #ifndef NEWGUI
 
@@ -281,6 +286,7 @@ extern bool load_world(const char *mname, const char *cname = NULL);
 extern bool save_world(const char *mname, bool nolms = false, bool octa = false);
 extern void getmapfilenames(const char *cname, bool fall = true);
 extern uint getmapcrc();
+extern bool loadents(const char *fname, vector<entity> &ents, uint *crc = NULL);
 
 // physics
 extern void moveplayer(physent *pl, int moveres, bool local);
